﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Data;
using Microsoft.Synchronization.Data.SqlExpress;
using Microsoft.Synchronization.Data;

namespace SQLChangeTrackingSync.Sample.Providers
{
    // this would likely have concrete implementation, and not allow anchor customization.
    public class SampleClientSyncProvider : SqlExpressClientSyncProvider
    {
        public SampleClientSyncProvider(string anchorSchema, string anchorTable)
        {
            InitializeAnchorCommands(anchorSchema, anchorTable);
        }

        private string _anchorSchema = "sync";
        private string _anchorTable = "Anchor";

        protected virtual void InitializeAnchorCommands(string anchorSchema, string anchorTable)
        {
            _anchorSchema = anchorSchema;
            _anchorTable = anchorTable;

            //-- Variable Prep
            string currentTableParameter = string.Format("@{0}", SqlExpressClientSyncProvider.PARAMETER_CURRENT_TABLE);
            string anchorValueParameter = string.Format("@{0}", SqlExpressClientSyncProvider.PARAMETER_ANCHOR_VALUE);
            string newAnchorParameter = string.Format("@{0}", SyncSession.SyncNewReceivedAnchor);
            string anchorServerAppliedParameter = string.Format("@{0}", SqlExpressClientSyncProvider.PARAMETER_ANCHOR_SERVER_APPLIED);

            //-- Anchor Command
            SqlCommand selectNewAnchorCommand = new SqlCommand(string.Format("SELECT {0} = change_tracking_current_version()", newAnchorParameter));
            selectNewAnchorCommand.Parameters.Add(newAnchorParameter, SqlDbType.BigInt);
            selectNewAnchorCommand.Parameters[newAnchorParameter].Direction = ParameterDirection.Output;
            this.SelectNewAnchorCommand = selectNewAnchorCommand;

            //-- Get Table Received
            SqlCommand tableReceivedCommand = new SqlCommand(string.Format("SELECT [anc_received] FROM [{0}].[{1}] WHERE [anc_table] = {2}", anchorSchema, anchorTable, currentTableParameter));
            tableReceivedCommand.Parameters.Add(currentTableParameter, SqlDbType.NVarChar, 128);
            this.SelectTableReceivedAnchorCommand = tableReceivedCommand;

            //-- Set Table Received
            SqlCommand updateReceivedCommand = new SqlCommand(string.Format("UPDATE [{0}].[{1}] SET [anc_received] = {2} WHERE [anc_table] = {3}", anchorSchema, anchorTable, anchorValueParameter, currentTableParameter));
            updateReceivedCommand.Parameters.Add(anchorValueParameter, SqlDbType.BigInt);
            updateReceivedCommand.Parameters.Add(currentTableParameter, SqlDbType.NVarChar, 128);
            this.UpdateTableReceivedAnchorCommand = updateReceivedCommand;

            //-- Get Table Sent
            SqlCommand tableSentCommand = new SqlCommand(string.Format("SELECT [anc_sent] FROM [{0}].[{1}] WHERE [anc_table] = {2}", anchorSchema, anchorTable, currentTableParameter));
            tableSentCommand.Parameters.Add(currentTableParameter, SqlDbType.NVarChar, 128);
            this.SelectTableSentAnchorCommand = tableSentCommand;


            //-- Set Table Sent
            SqlCommand updateSentCommand = new SqlCommand(string.Format("UPDATE [{0}].[{1}] SET [anc_sent] = {2} WHERE [anc_table] = {3}", anchorSchema, anchorTable, anchorValueParameter, currentTableParameter));
            updateSentCommand.Parameters.Add(anchorValueParameter, SqlDbType.BigInt);
            updateSentCommand.Parameters.Add(currentTableParameter, SqlDbType.NVarChar, 128);
            this.UpdateTableSentAnchorCommand = updateSentCommand;

            //-- Set Table Server Applied
            SqlCommand updateServerAppliedCommand = new SqlCommand(string.Format("UPDATE [{0}].[{1}] SET [anc_server_applied] = {2} WHERE [anc_table] = {3}", anchorSchema, anchorTable, anchorServerAppliedParameter, currentTableParameter));
            updateServerAppliedCommand.Parameters.Add(currentTableParameter, SqlDbType.NVarChar, 128); // it can't auto-add it (because it operates at abstract level)
            updateServerAppliedCommand.Parameters.Add(anchorServerAppliedParameter, SqlDbType.BigInt); // it can't auto-add it (because it operates at abstract level)
            this.UpdateTableServerAppliedAnchorCommand = updateServerAppliedCommand;

        }


    }
}
