﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Synchronization.Data.Server;
using Microsoft.Synchronization.Data;
using System.Data.SqlClient;
using System.Data;
using SQLChangeTrackingSync.Sample.Entities;

namespace SQLChangeTrackingSync.Sample.Providers
{
    public class SampleServerSyncProvider : DbServerSyncProvider
    {
        #region Constructor

        public SampleServerSyncProvider()
        {
            InitializeAnchorCommands();
        }

        #endregion

        #region Private Properties

        private Guid _requestingClientID;
        private Guid _serverID;
        private Dictionary<string, ApplyAction> _syncPolicy;
        private bool _authorized;

        #endregion

        #region Public Properties

        public Guid ServerID
        {
            get
            {
                return _serverID;
            }
            set
            {
                _serverID = value;
            }
        }
        public Guid RequestingClientID
        {
            get
            {
                return _requestingClientID;
            }
            set
            {
                _requestingClientID = value;
            }
        }

        #endregion

        #region Public Methods

        public bool Authorize(Guid clientID)//TODO: Adjust signature to match required data
        {
            //TODO: Get policy
            //TODO: Authenticate | Authorize
            _authorized = true;
            if (_authorized)
            {
                _syncPolicy = new Dictionary<string, ApplyAction>(StringComparer.OrdinalIgnoreCase);
                this.ApplyChangeFailed += new EventHandler<ApplyChangeFailedEventArgs>(SampleServerSyncProvider_ApplyChangeFailed);
            }
            return _authorized;
        }

        #endregion

        #region Protected Methods

        protected virtual void InitializeAnchorCommands()
        {
            string newAnchorVariable = "@" + SyncSession.SyncNewReceivedAnchor;
            SqlCommand selectNewAnchorCommand = new SqlCommand(string.Format("SELECT {0} = change_tracking_current_version()", newAnchorVariable));
            selectNewAnchorCommand.Parameters.Add(newAnchorVariable, SqlDbType.BigInt);
            selectNewAnchorCommand.Parameters[newAnchorVariable].Direction = ParameterDirection.Output;
            this.SelectNewAnchorCommand = selectNewAnchorCommand;
        }
        protected virtual ApplyAction GetPolicyAction(ConflictType conflictType, ApplyAction defaultAction)
        {
            ApplyAction result = defaultAction;
            if (_syncPolicy.ContainsKey(conflictType.ToString()))
            {
                result = _syncPolicy[conflictType.ToString()];
            }
            return result;
        }
        protected virtual void HandleApplyChangeFailed(ApplyChangeFailedEventArgs e)
        {
            if (!e.Session.SyncParameters.Contains("Conflicts"))
            {
                e.Session.SyncParameters.Add("Conflicts", new List<ConflictLog>());
            }
            List<ConflictLog> conflicts = e.Session.SyncParameters["Conflicts"].Value as List<ConflictLog>;

            ConflictLog conflict = null;
            switch (e.Conflict.ConflictType)
            {
                case ConflictType.ClientDeleteServerUpdate:
                    // Server Wins (From Policy) Notify User, shouldn't happen based on delete architecture
                    e.Action = GetPolicyAction(e.Conflict.ConflictType, ApplyAction.Continue);
                    conflict = new ConflictLog(string.Format("A local item was restored from the server. (A recent delete may have been undone). [{0}]. Server chose to: {1}.", e.TableMetadata.TableName, e.Action));
                    conflict.Detail.Right = SyncUtility.ConvertToXml(e.Conflict.ServerChange);
                    break;
                case ConflictType.ClientInsertServerInsert:
                    // Shouldn't happen based on distributed keys, Server wins, notify user.
                    e.Action = GetPolicyAction(e.Conflict.ConflictType, ApplyAction.Continue);
                    conflict = new ConflictLog(string.Format("Duplicate Primary Keys were detected, The local versions were overwritten. Please notify your administrator. [{0}]. Server chose to: {1}.", e.TableMetadata.TableName, e.Action));
                    break;
                case ConflictType.ClientUpdateServerDelete:
                    // Server Wins (From Policy) Notify User, shouldn't happen based on delete architecture
                    e.Action = GetPolicyAction(e.Conflict.ConflictType, ApplyAction.Continue);
                    conflict = new ConflictLog(string.Format("A local item was deleted by the server. (A recent update may have been lost). [{0}]. Server chose to: {1}.", e.TableMetadata.TableName, e.Action));
                    conflict.Detail.Left = SyncUtility.ConvertToXml(e.Conflict.ClientChange);
                    break;
                case ConflictType.ClientUpdateServerUpdate:
                    // Server Wins (From Policy) Notify User
                    e.Action = GetPolicyAction(e.Conflict.ConflictType, ApplyAction.Continue);
                    conflict = new ConflictLog(string.Format("A local updated item was updated by the server. (Your update will be lost). [{0}]. Server chose to: {1}.", e.TableMetadata.TableName, e.Action));
                    conflict.Detail.Left = SyncUtility.ConvertToXml(e.Conflict.ClientChange);
                    conflict.Detail.Right = SyncUtility.ConvertToXml(e.Conflict.ServerChange);
                    break;
                case ConflictType.ErrorsOccurred:
                    // Notify User, Cancel
                    conflict = new ConflictLog(string.Format("An unhandled error occured, sync may stop: {0}", e.Conflict.ErrorMessage));
                    break;
                case ConflictType.Unknown:
                default:
                    // Notify User, Cancel
                    conflict = new ConflictLog("An unknown conflict occured, sync may stop.");
                    break;
            }

            conflicts.Add(conflict);
        }

        #endregion

        #region Event Handlers

        private void SampleServerSyncProvider_ApplyChangeFailed(object sender, ApplyChangeFailedEventArgs e)
        {
            HandleApplyChangeFailed(e);
        }

        #endregion

    }
}
