SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE DATABASE [SyncDemoClient]
GO

USE [SyncDemoClient]
GO


-- Standard Elements

CREATE TABLE [dbo].[Anchor](
	[anc_table] [nvarchar](128) NOT NULL,
	[anc_sent] [bigint] NULL,
	[anc_received] [bigint] NULL,
	[anc_server_applied] [bigint] NULL,
 CONSTRAINT [PK_sync_Anchor] PRIMARY KEY CLUSTERED 
(
	[anc_table] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO



CREATE TABLE [dbo].[KeyGeneration](
	[pk_client] [int] NOT NULL,
	[pk_table_name] [nvarchar](128) NOT NULL,
	[pk_client_name] [nvarchar](50) NOT NULL,
	[pk_row_id_local] [int] NOT NULL,
	[pk_row_id_global]  AS (isnull([pk_client]*power(CONVERT([bigint],(2),(0)),(32))+[pk_row_id_local],(0))),
 CONSTRAINT [PK_KeyGeneration] PRIMARY KEY CLUSTERED 
(
	[pk_client] ASC,
	[pk_table_name] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO

CREATE TABLE [dbo].[FakeTable](
	[fkt_id] [bigint] NOT NULL,
	[fkt_name] [nvarchar](50) NOT NULL,
	[fkt_desc] [nvarchar](255) NULL,
 CONSTRAINT [PK_FakeTable] PRIMARY KEY CLUSTERED 
(
	[fkt_id] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]

GO


-- Primary Key Helpers

CREATE PROCEDURE [dbo].[spGenerateKey]
	@ClientID int
	,@TableName nvarchar(128)
	,@RowIDGlobal bigint output
AS
BEGIN

	SET NOCOUNT ON;
	
	UPDATE [dbo].[KeyGeneration]
		SET [pk_row_id_local] += 1,
			@RowIDGlobal = [pk_row_id_global]
	WHERE [pk_client] = @ClientID AND [pk_table_name] = @TableName;

	
END




-- Change Tracking

DECLARE @RemoveOnly BIT
SET @RemoveOnly = 0

-- First Pass
IF (SELECT snapshot_isolation_state from sys.databases where name = 'SyncDemoClient') = 0 BEGIN
	IF @RemoveOnly = 1 BEGIN
		SELECT N'Snapshot Isolation was already OFF. Nothing to remove.' as [Status]
	END ELSE BEGIN
		ALTER DATABASE [SyncDemoClient] SET ALLOW_SNAPSHOT_ISOLATION ON
		SELECT N'Snapshot Isolation was changed to ON' as [Status]
	END
END ELSE BEGIN
	IF @RemoveOnly = 1 BEGIN
		SELECT N'Snapshot Isolation will be set to OFF after other processes complete.' as [Status]
	END ELSE BEGIN
		SELECT N'Snapshot Isolation is already ON' as [Status]
	END
END



IF EXISTS(SELECT * from sys.change_tracking_databases where [database_id] = DB_ID('SyncDemoClient') ) BEGIN
	IF @RemoveOnly = 1 BEGIN
		SELECT N'Change Tracking will be set to OFF after other processes complete.' as [Status]
	END ELSE BEGIN
		SELECT N'Change Tracking was already configured' as [Status]
	END
END ELSE BEGIN
	IF @RemoveOnly = 1 BEGIN
		SELECT N'Change Tracking was already OFF. Nothing to remove.' as [Status]
	END ELSE BEGIN
		ALTER DATABASE [SyncDemoClient]
		SET CHANGE_TRACKING = ON
		(CHANGE_RETENTION = 14 DAYS, AUTO_CLEANUP = ON)
		SELECT N'Change Tracking was added' as [Status]
	END
END


IF NOT EXISTS(select * from  sys.change_tracking_tables WHERE [object_id] = OBJECT_ID(N'dbo.FakeTable')) BEGIN
	IF @RemoveOnly = 1 BEGIN
		SELECT N'Change Tracking was not present. Nothing to remove.' as [Status]
	END ELSE BEGIN
		ALTER TABLE [dbo].[FakeTable]
		ENABLE CHANGE_TRACKING
		WITH (TRACK_COLUMNS_UPDATED = ON)
		SELECT N'Change Tracking was added to table [dbo].[FakeTable]' as [Status]
	END
END ELSE BEGIN
	IF @RemoveOnly = 1 BEGIN
		ALTER TABLE [dbo].[FakeTable]
		DISABLE CHANGE_TRACKING
		SELECT N'Change Tracking was removed from table [dbo].[FakeTable]' as [Status]
	END ELSE BEGIN
		SELECT N'Change Tracking was already active on table [dbo].[FakeTable]' as [Status]
	END
END


-- Second Pass

IF EXISTS(SELECT * from sys.change_tracking_databases where [database_id] = DB_ID('SyncDemoClient') ) BEGIN
	IF @RemoveOnly = 1 BEGIN
		ALTER DATABASE [SyncDemoClient]
		SET CHANGE_TRACKING = OFF
		SELECT N'Change Tracking was disabled' as [Status]
	END
END 
IF (SELECT snapshot_isolation_state from sys.databases where name = 'SyncDemoClient') = 1 BEGIN
	IF @RemoveOnly = 1 BEGIN
		ALTER DATABASE [SyncDemoClient] SET ALLOW_SNAPSHOT_ISOLATION OFF
		SELECT N'Snapshot Isolation was set to OFF.' as [Status]
	END
END


